// Camera.h
//		Camera class intended for use with OpenGL ES 2.0
//		Compiled 2/10/2011 by Matt Pierce
#pragma once
#include "VectorMath.h"		// Custom vector math library.
							// Any distributions of this class must also contain a copy of VectorMath.h and VectorMath.cpp

// Camera class objecct
class CCamera {
	private:
		SVect3d pos, lookat, up;		// these variables are user-configured through accessors and correspond to the arguments of gluLookAt
		SVect3d vfwd, vup, vright;		// these variables hold data used to generate the transformation matrix
		float roll;
		
		void UpdateVectors();			// an internal function that calculates values for vfwd, vup, and vright
		
	public:
		CCamera() {						// constructor - creates default OpenGL camera at Origin facing -Z
			pos = v3d(0,0,0);
			lookat = v3d(0, 0, -1);
			up = v3d(0, 1, 0);
			roll = 0;
			UpdateVectors();
		};
		~CCamera() {};					// destructor - no allocated data
		
		void GetMatrixArray(float *tMat);			// return a matrix compatible with OpenGL ES 2.0 mat4/glUniformMatrix4fv
		
		void SetPos(const SVect3d &aPos);			// set camera position
		void SetLookAt(const SVect3d &aLookAt);		// set the coordinate the camera faces
		void SetUp(const SVect3d &aUp);				// set the vector describing what's "up" from our view (controls roll)
		void SetRoll(const float &aRoll);			// set a roll amount independent from "up" - mostly for orientation changes
		
		SVect3d GetPos() {return pos;};
		SVect3d GetLookAt() {return lookat;};
		SVect3d GetUp() {return up;};
		float GetRoll() { return roll;};
		
};

// Standard GL perspective matrix creation
void Persp(float Proj[4][4], const float w, const float h, const float FOV, const float ZNear, const float ZFar);

// Standard GL orthographic matrix creation
void Ortho(float Ortho[4][4], const float left, const float right, const float bottom, const float top, const float nearVal, const float farVal);


